#!/usr/bin/perl 

# 
#  FASt Term Recognizer
# 
#   DISTRIB/bin/CelexPreProc.pl
#   Version 2.04/04-02-01
# 
#   Copyright (C) 2004  Christian Jacquemin, LIMSI-CNRS
# 
#   Pre-processing of the Celex database for creating morphological families
#   this version discards compounds (with an internal blank) from the
#   families (search in the source code for COMPOUNDS)

# IN: FILElemmas = lemmas with a derivational history (EML)
#
# 91\abort\35\R\\0\N\N\N\N\N\\\\\\\\\\\\\\
# 92\abortion\351\C\\1\N\N\N\N\Y\abort+ion\3x\SA\N\N\N\#\N\N\SA\
# ((abort)[V],(ion)[N|V.])[N]\N\N\N
# 93\abortionist\18\C\\1\N\N\N\N\Y\abortion+ist\Nx\SA\N\N\N\#\N\N\SAA\
# (((abort)[V],(ion)[N|V.])[N],(ist)[N|N.])[N]\N\N\N
# 94\abortive\59\C\\1\N\N\N\N\Y\abortion+ive\Nx\SA\N\Y\N\-ion#\N\N\SAA\
# (((abort)[V],(ion)[N|V.])[N],(ive)[A|N.])[A]\N\Y\N
# 95\abortively\0\C\\1\N\N\N\N\Y\abortive+ly\Ax\SA\N\N\N\#\N\N\SAAA\
# ((((abort)[V],(ion)[N|V.])[N],(ive)[A|N.])[A],(ly)[B|A.])[B]\N\Y\N

# IN: FILEsynt = part-of-speech categories (ESL)
# 91\abort\35\4\N\N\N\N\N\N\N\N\N\N\N\Y\N\Y\N\N\N\N\N\N\N\N\N\N\N
# \N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N
# 92\abortion\351\1\Y\Y\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N
# \N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N
# 93\abortionist\18\1\Y\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N
# \N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N
# 94\abortive\59\2\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\Y\N\N\N
# \N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N
# 95\abortively\0\7\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N
# \N\N\Y\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N\N

# STDOUT: morphological families
# 10291\crayon\N\: 10291\crayon\N\-+ 10292\crayon\V\-+
# 10293\craze\N\: 10293\craze\N\-+ 10296\crazily\ADV\-y+i+ly 
#                 10297\craziness\N\-y+i+ness 10298\crazy\A\-e+y
# 10301\creak\V\: 10300\creak\N\-+ 10301\creak\V\-+ 
#                 10302\creakily\ADV\-y+i+ly 10303\creakiness\N\-y+i+ness 
#                 10304\creaky\A\-+y

# syntactic check
if( (@ARGV + 0) != 2 ) {
    die "Usage: CelexPreProc.pl <Celex-path>/EML/eml.cd <Celex-path>/ESL/esl.cd!\n";
}

$NAMEFILE = $ARGV[0];
open( FILElemmas , "<$NAMEFILE" ) ||  die "Can't open file $NAMEFILE!" ;
$NAMEFILE = $ARGV[1];
open( FILEsynt , "<$NAMEFILE" ) ||  die "Can't open file $NAMEFILE!" ;

  # strings associated to field 3 in ESL
  $CLArray[1] = "N";
  $CLArray[2] = "A";   
  $CLArray[3] = "NUM";   
  $CLArray[4] = "V";   
  $CLArray[5] = "ART";   
  $CLArray[6] = "PRON";   
  $CLArray[7] = "ADV";   
  $CLArray[8] = "PREP";   
  $CLArray[9] = "C";   
  $CLArray[10] = "I";
  $CLArray[11] = "SCON";
  $CLArray[12] = "CCON";
  $CLArray[13] = "LET";
  $CLArray[14] = "ABB";   
  $CLArray[15] = "TO";

  # transformation of the categories of the derivational history into
  # normalized categories
  $catNormal{ "N" } = "N";
  $catNormal{ "A" } = "A";
  $catNormal{ "V" } = "V";
  $catNormal{ "B" } = "ADV";
  $catNormal{ "P" } = "PREP";
  $catNormal{ "I" } = "I";
  $catNormal{ "O" } = "PRON";
  $catNormal{ "Q" } = "NUM";
  $catNormal{ "C" } = "C";

# reads the categories
while(<FILEsynt>) {
    @fields = split /\\/;
    $ref = $fields[0];
    $cat{ $ref } = $CLArray[ $fields[3] ];
}
close( FILEsynt );

# builds the families
while(<FILElemmas>) {
    @fields = split /\\/;
    $derivedref = $fields[0];
    $derivedlemma = $fields[1];

    # affix extraction
    $affixes = $fields[11];
    if( $affixes =~ /\+.*\+/ ) {
	$affixes =~ s/\+.*\+/\+\+/;
    }
    elsif( $affixes =~ /\+/ ) {
	$affixes =~ s/.*\+/\+/;
    }
    else {
	$affixes = "+";
    }
    $sufStripping = $fields[17];
    if( $sufStripping =~ /\-/ ) {
	chop( $sufStripping );
    }
    else {
	$sufStripping = "-";
    }

    # caring about the derivational history
    $history = $fields[21];
    # COMPOUNDS are discarded here 
    if( $derivedlemma =~ / / ) {
	next;
    }

    # discarding lemmas without history
    if( !$history ) {
	next;
    }

    # getting the root
    # case 1: word with multiple roots
    if( $history =~ /\(([^\)\(]+)\)\[([^\]\|]+)\][^,]*$/ ){
	# the head word is the last word for compounds
	$lemma = $1;
	$category = $catNormal{ $2 };
	
	if( $category ne "PREP" ) {
	    $history = 
		"(" . $lemma . ")[" . $category . "]";
	}
	# the head word is the first word if the last word is a
	# prep (eg V P )
	elsif( $history =~ /\(([^\)\(]+)\)\[([^\]\|]+)\]/ ){
	    # the head word is the last word for compounds
	    $lemma = $1;
	    $category = $catNormal{ $2 };
	    $history = 
		"(" . $lemma . ")[" . $category . "]";
	}
    # case 2: word with a single root
    } elsif( $history =~ /\(([^\)\(]+)\)\[([^\]\|]+)\]/ ){
	# the head word is the last word for compounds
	$lemma = $1;
	$category = $catNormal{ $2 };
	$history = 
	    "(" . $lemma . ")[" . $category . "]";
    }

    # adding a lemma to its own family is not necessary because 
    # it will be automatically done during the following post-processing
    else {
	next;
    }

    # the history of the current lemma has not been built
    if( !$history ){
	next;
    }

    # adding the current lemma to the family of its root
    $structuredLemma = "(" . $derivedlemma . ")[" 
	. $cat{ $derivedref } . "]";
    $affixes{ $structuredLemma } = $sufStripping . $affixes;
    if( $family{ $history } ){
	$family{ $history } = $family{ $history } . "\t" . $structuredLemma;
    }
    else{
	$family{ $history } = $structuredLemma;
    }
}

# adding a lemma to its own family and dropping the single item families
foreach $key (keys %family){
    @fields = split /\t/ , $family{ $key };
    $found = 0;
    # is the item member of its own family?
    foreach $lemma (@fields){
	if( $lemma eq $key ){
	    $found = 1;
	    last;
	}
    }
    # no, if it has a known reference then add it
    if( ! $found ) {
	$family{ $key } = $family{ $key } . "\t" . $key;
	push( @fields , $key );
	$affixes{ $key } = "-+";
    }
    # delete single-item families
    if( @fields - 1 ){
	if( $key =~ /\((.+)\)\[(.*)\]$/ ){
	    print $1 , "\\" , $2 , "\\:";
	    foreach $structuredLemma (@fields){
		if( $structuredLemma =~ /\((.+)\)\[(.*)\]$/ ){
		    print " " , $1 , "\\" , $2 , "\\" , 
		           $affixes{ $structuredLemma };
		}
	    }
	    print "\n";
	}
    }
    else {
	delete( $family{ $key } );
    }
}

close( FILElemmas );



