#!/usr/bin/perl 

# 
#  FASt Term Recognizer
# 
#   fastr/bin/TermerforFastr.pl
#   Version 2.04/04-02-01
# 
#   Copyright (C) 2004  Christian Jacquemin, LIMSI-CNRS
#   BP 133, 91403 ORSAY, FRANCE 
#   tel +33 (0)1 69 85 80 22 / fax -- 80 88 
#   http://www.limsi.fr/Individu/jacquemi/ 
# 
#   Term acquisition from a tagged corpus (simplistic)
# 

# STDIN: a corpus tagged by TreeTagger (tree-tagger-english)
#
# Since   IN      since
# large   JJ      large
# amounts NNS     amount
# of      IN      of
# textual JJ      textual


# STDOUT: candidate terms
#
# 00000[\t]domain document [\t]NN NNS 
# 00001[\t]entire sentence [\t]JJ NNS 
# 00002[\t]high information [\t]JJ NN 

# syntactic check
if( (@ARGV + 0) != 1 ) {
    die "Usage: TermerforFastr.pl [-en|-fr]!\n";
}

# reading the language option
# and defining the acquisition pattern
$language = shift @ARGV;
if( $language eq "-en" ) {
    $Pattern = "((JJ|NN|NP|VBD|VBG) (NN|NNS))";
}
elsif( $language eq "-fr" ) {
    $Pattern = "((NOM PRP (DET(\:[a-z]+)? )?NOM)|(NOM ADJ)|(ADJ NOM))";
}
else {
    die "Usage: TermerforFastr.pl [-en|-fr]!\n";
}

$nbWords = 0;
@bufferTags = ();
@bufferInflWords = ();
@bufferLemmas = ();
# parsing the tagged file
while (<>) {
    chop;

    # empty sentence or empty term description
    if( /^$/ ){
    }

    # document identifier
    if( /^\<.*\>$/ ){
	$INFLWORD = $_;
	$LEMMA = $_;
	push( @bufferTags , "Struct" );
	push( @bufferInflWords , $INFLWORD );
	push( @bufferLemmas , $LEMMA );
    }

    # splitting a line of the tagger
    if( !( /^\<.*\>$/ )  && !( /^$/ ) ){
	( $INFLWORD , $TAG , $LEMMA ) = split( /\t/ );

	# unknown lemmas are replaced by the inflected word
	if( $LEMMA eq "<unknown>" ) {
	    $LEMMA = $INFLWORD;
	}

	push( @bufferTags , $TAG );
	push( @bufferInflWords , $INFLWORD );
	push( @bufferLemmas , $LEMMA );
    }

    # joining words in sequence
    $tagSequence = join( " " , @bufferTags );

    &termAcquisition;

    # first in first out queue
    if( $nbWords >= 8 ) {
	shift( @bufferTags );
	shift( @bufferInflWords );
	shift( @bufferLemmas );
	$nbWords--;
    }
    $nbWords++;
}

# processing of the trailing words
while( $nbWords >= 2 ) {
    # joining words in sequence
    $tagSequence = join( " " , @bufferTags );

    &termAcquisition;

    # first in first out queue
    shift( @bufferTags );
    shift( @bufferInflWords );
    shift( @bufferLemmas );
    $nbWords--;
}

# term acquisition
sub termAcquisition {
    if( $tagSequence =~ /^$Pattern/ ) {
	$termTagSequence = $1;
	@termTagSequence = split( / / , $termTagSequence );

	# prints the lemmas of the term
	$i = 0;
	foreach $tag ( @termTagSequence ) {
	    printf "%s " , @bufferLemmas[ $i ];
	    $i++;
	}
	print "\t";

	# prints the pos tags of the term
	$i = 0;
	foreach $tag ( @termTagSequence ) {
	    printf "%s " , @bufferTags[ $i ];
	    $i++;
	}
	print "\n";
    }   
}
