#!/usr/bin/perl 
 
# 
#  FASt Term Recognizer
# 
#   DISTRIB/bin/TreeTaggertoTerms.pl
#   Version 2.04/04-02-01
# 
#   Copyright (C) 2004  Christian Jacquemin, LIMSI-CNRS
# 
#   Conversion of terms tagged by TreeTagger 
#   into candidate terms
# 

# STDIN: Terms tagged by TreeTagger (tree-tagger-english)
#
# biologie        NOM     biologie
# intgre        VER:pper        intgrer
# .       PON:sep .
# biophysique     NOM     biophysique
# .       PON:sep .
# biosystmatique NOM     <unknown>
# .       PON:sep .

# STDOUT: candidate terms
#
# domain document [\t]NN NNS 
# entire sentence [\t]JJ NNS 
# high information [\t]JJ NN 

# syntactic check
if( (@ARGV + 0) != 0 ) {
    die "Usage: TreeTaggertoTerms.pl!\n";
}

# converting the tagged file
while (<>) {
    chop;

    # splitting the line into: inflected word, tag, and lemma
    ( $INFLWORD , $TAG , $LEMMA ) = split( /\t/ );
    
    # unknown lemmas are replaced by the inflected word
    if( $LEMMA eq "<unknown>" || $LEMMA eq "\@card\@" ) {
	$LEMMA = $INFLWORD;
    }
    
    # normalizing words
    if( !($TAG =~ /[\#\$\'()\.:;,]/ ) ) {
	&CleanLemma;
    }

    # dot finishing a term: prints the term
    if( /^\./ ) {
	foreach $lemma ( @lemmas ) {
	    print $lemma , " " ;
	}
	print "\t";
	foreach $tag ( @tags ) {
	    print $tag , " " ;
	}
	print "\n";
	@lemmas = ();
	@tags = ();
    }
    else {
	push( @tags , $TAG );
	push( @lemmas , $LEMMA );
    }
}

sub CleanLemma {
    # suppression of parasite characters
    $INFLLEMMA =~ s/[\"\{\}\$\~\=\|*,\\\]\[\`\<\>\?\@\_]//g;
    # deletion of double quotes
    $INFLLEMMA =~ s/\'\'//g;
    # deletion of initial and final single quote
    $INFLLEMMA =~ s/^\'//;
    $INFLLEMMA =~ s/\'$//;
    # normalization of double or triple hyphens
    $INFLLEMMA =~ s/---/-/g;
    $INFLLEMMA =~ s/--/-/g;
    # deletion of initial and final  hyphen
    $INFLLEMMA =~ s/^-//;
    $INFLLEMMA =~ s/-$//;
}
